<?php
App::uses('EvSitemapAppController', 'EvSitemap.Controller');

class SitemapsController extends EvSitemapAppController {

	public $adminActions = [
		'admin_index'
	];

/**
 * Constructor.
 *
 * @param CakeRequest $request Request object for this controller. Can be null for testing,
 *  but expect that features that use the request parameters will not work.
 * @param CakeResponse $response Response object for this controller.
 */
	public function __construct($request = null, $response = null) {
		/*
		 * Controller would normally be added from EvCore but as the plugin doesn't require EvCore and for fallback
		 * purposes, add the required helpers and components here.
		 */
		if (!isset($this->components['RequestHandler']) && !in_array('RequestHandler', $this->components)) {
			$this->components[] = 'RequestHandler';
		}

		if (!isset($this->helpers['Html']) && !in_array('Html', $this->helpers)) {
			$this->helpers[] = 'Html';
		}

		if (!isset($this->helpers['Time']) && !in_array('Time', $this->helpers)) {
			$this->helpers[] = 'Time';
		}

		parent::__construct($request, $response);
	}

/**
 * Before filter
 *
 * @return void
 */
	public function beforeFilter() {
		parent::beforeFilter();

		$this->Auth->allow(array(
			'index'
		));
	}

/**
 * Render the sitemap
 *
 * @return void
 */
	public function index() {
		$listing = array();

		$pageId = Configure::read('EvSitemap.page_id');

		if (! empty($pageId)) {
			$this->loadModel('Page');
			$this->set('data', $this->Page->readForView($pageId));
		}

		$cacheName = Configure::read('EvSitemap.cacheName');
		$requestType = $this->request->isXml() ? 'xml' : 'html';
		$listing = null;

		// If cacheName is defined try get the listing from the cache
		if (!empty($cacheName)) {
			$listing = Cache::read('EvSitemap.listing.' . $requestType, $cacheName);
		}

		if (empty($listing)) {
			// assign list of models to be used within the sitemap
			$modelListing = $this->_getModelListing();

			if (! empty($modelListing)) {
				foreach ($modelListing as $modelName) {

					list(, $model) = pluginSplit($modelName);

					try {

						$this->loadModel(ucwords($modelName));
						$Model = $this->{$model};

					} catch(\Exception $e) {
						continue;
					}

					if (!empty($Model->actsAs) && $Model->hasBehavior('Sitemap')) {
						$key = ! empty($Model->actsAs['Sitemap']['listingTitle']) ? $Model->actsAs['Sitemap']['listingTitle'] : Inflector::humanize(Inflector::underscore(Inflector::pluralize($Model->name)));
						$listing[$key] = $Model->getSitemapData();
					}

					unset($Model);
				}
			}

			// Write back to the cache if defined
			if (!empty($cacheName)) {
				Cache::write('EvSitemap.listing.' . $requestType, $listing, $cacheName);
			}
		}

		$this->set(compact('listing'));

		return;
	}

	public function admin_index() {
		if (empty(Configure::read('EvSitemap.upload.enabled'))) {
			throw new NotImplementedException('You must enable sitemap upload by setting EvSitemap.upload.enabled to true!');
		}

		if ($this->request->is('post')) {
			$data = $this->request->data;
			if (empty($data['Sitemaps']['sitemap']['name'])) {
				$this->Flash->fail([
					'title' => 'Upload failed',
					'description' => 'You must upload a file!'
				]);
			} elseif ($data['Sitemaps']['sitemap']['type'] !== 'text/xml') {
				$this->Flash->fail([
					'title' => 'Upload failed',
					'description' => 'Your file must have the .xml extension'
				]);
			} elseif (move_uploaded_file($data['Sitemaps']['sitemap']['tmp_name'], WWW_ROOT . 'sitemap.xml')) {
				$this->Flash->success(
					[
						'title' => 'Upload Successful',
						'description' => 'Your sitemap has been successfully uploaded'
					]);
			}

			return $this->redirect($this->here);
		}
	}

/**
 * Clear the cache and redirect to the sitemap
 *
 * @return void
 */
	public function refresh() {
		$cacheName = Configure::read('EvSitemap.cacheName');

		if (!empty($cacheName)) {
			Cache::delete('EvSitemap.listing.html', $cacheName);
			Cache::delete('EvSitemap.listing.xml', $cacheName);
		}

		return $this->redirect(['action' => 'index']);
	}

/**
 * Fetches the list of models to check
 *
 * @return array A list of models
 */
	protected function _getModelListing() {
		$listing = array();

		$ignoreList = Configure::read('EvSitemap.modelIgnoreList');

		// Get all the models in the main app model directory
		$modelList = App::objects('Model');

		// get all the plugins so we can get all their models
		$pluginList = App::objects('plugins');

		// loop and get the models
		foreach ($pluginList as $plugin) {

			// get the plugin models and prefix with the plugin name
			$pluginModel = App::objects($plugin . '.Model');

			foreach ($pluginModel as $key => $model) {

				$pluginModel[$key] = $plugin . '.' . $model;
			}

			$modelList = array_merge($modelList, $pluginModel);
		}

		if (! empty($modelList)) {

			foreach ($modelList as $modelName) {

				if (in_array($modelName, $ignoreList)) {

					continue;
				}

				$listing[] = $modelName;
			}
		}

		return $listing;
	}

}
